#include "roomlistmodel.h"

#include <algorithm>

static auto byTag = [](QSharedPointer<Room> a, QSharedPointer<Room> b) { return a->tag < b->tag; };

QModelIndex RoomListModel::calcIndex(std::string id, const std::vector<QSharedPointer<Room>> &invites,
                                     const std::vector<QSharedPointer<Room>> &other) {
    int pos = 0;
    for (const auto &r : invites) {
        if (r->id == id)
            return index(pos, 0);
        else
            pos++;
    }
    for (const auto &r : other) {
        if (r->id == id)
            return index(pos, 0);
        else
            pos++;
    }

    return index(-1, 0);
}

void RoomListModel::addRoom(QSharedPointer<Room> room) {
    auto pos = std::upper_bound(std::begin(joinedRooms), std::end(joinedRooms), room, byTag);
    int posIdx = (int)invites.size() + std::distance(joinedRooms.begin(), pos);
    beginInsertRows(QModelIndex(), posIdx, posIdx);
    joinedRooms.insert(pos, room);
    endInsertRows();

    rooms[room->id] = room;
}

void RoomListModel::addInvite(QSharedPointer<Room> room) {
    beginInsertRows(QModelIndex(), (int)invites.size(), (int)invites.size());
    invites.push_back(room);
    endInsertRows();

    rooms[room->id] = room;
}

void RoomListModel::updateRoom(QSharedPointer<Room> room) {
    rooms[room->id] = room;

    auto idx = calcIndex(room->id, invites, joinedRooms);
    if (idx.row() != -1)
        emit dataChanged(idx, idx);
}

QHash<int, QByteArray> RoomListModel::roleNames() const {
    QHash<int, QByteArray> roles;
    roles[RoomNameRole] = "roomName";
    roles[UnreadNotificationsRole] = "unreadNotifications";
    roles[LastMessageRole] = "lastMessage";
    roles[RoomPictureRole] = "roomPicture";
    roles[TagRole] = "tag";
    return roles;
}

int RoomListModel::rowCount(const QModelIndex &parent) const {
    Q_UNUSED(parent);
    return (int)invites.size() + joinedRooms.size();
}

QVariant RoomListModel::data(const QModelIndex &index, int role) const {

    if (index.row() < 0 || index.row() >= rowCount())
        return QVariant();

    const auto &room = [this, index]() {
        if (index.row() < (int)invites.size())
            return invites[index.row()];
        else
            return joinedRooms[index.row() - invites.size()];
    }();

    switch (role) {
    case RoomNameRole:
        return QString::fromStdString(room->name);
    case UnreadNotificationsRole:
        return room->unreadNotifications;
    case LastMessageRole:
        return room->msgInfo.body;
    case RoomPictureRole:
        return QString::fromStdString(room->avatar_url);
    case TagRole:
        return QString::fromStdString(room->tags.empty() ? "" : room->tags.front());

    default:
        return QVariant();
    }
}

RoomListModel::RoomListModel(QObject *parent) { Q_UNUSED(parent); }
